@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package Capture::Tiny;
 # ABSTRACT: Capture STDOUT and STDERR from Perl, XS or external programs
-our $VERSION = '0.29'; # TRIAL
+our $VERSION = '0.31'; # TRIAL
 use Carp ();
 use Exporter ();
 use IO::Handle ();
@@ -83,10 +83,18 @@ HERE
 sub _relayer {
   my ($fh, $layers) = @_;
   # _debug("# requested layers (@{$layers}) for @{[fileno $fh]}\n");
-  my %seen = ( unix => 1, perlio => 1 ); # filter these out
-  my @unique = grep { !$seen{$_}++ } @$layers;
-  # _debug("# applying unique layers (@unique) to @{[fileno $fh]}\n");
-  binmode($fh, join(":", ":raw", @unique));
+
+  # eliminate pseudo-layers
+  binmode( $fh, ":raw" );
+  # strip off real layers until only :unix is left
+  while ( 1 < ( my $layers =()= PerlIO::get_layers( $fh, output => 1 ) ) ) {
+      binmode( $fh, ":pop" );
+  }
+  # apply other layers
+  my @to_apply = @$layers;
+  shift @to_apply; # eliminate initial :unix
+  # _debug("# applying layers  (unix @to_apply) to @{[fileno $fh]}\n");
+  binmode($fh, ":" . join(":",@to_apply));
 }
 
 sub _name {
@@ -209,7 +217,9 @@ sub _start_tee {
   $stash->{flag_files}{$which} = scalar tmpnam();
   # execute @cmd as a separate process
   if ( $IS_WIN32 ) {
-    local $@;
+    my $old_eval_err=$@;
+    undef $@;
+
     eval "use Win32API::File qw/GetOsFHandle SetHandleInformation fileLastError HANDLE_FLAG_INHERIT INVALID_HANDLE_VALUE/ ";
     # _debug( "# Win32API::File loaded\n") unless $@;
     my $os_fhandle = GetOsFHandle( $stash->{tee}{$which} );
@@ -219,6 +229,7 @@ sub _start_tee {
     _open_std( $stash->{child}{$which} );
     $stash->{pid}{$which} = system(1, @cmd, $stash->{flag_files}{$which});
     # not restoring std here as it all gets redirected again shortly anyway
+    $@=$old_eval_err;
   }
   else { # use fork
     _fork_exec( $which, $stash );
@@ -358,19 +369,22 @@ sub _capture_tee {
   # _debug( "# redirecting in parent ...\n" );
   _open_std( $stash->{new} );
   # execute user provided code
-  my ($exit_code, $inner_error, $outer_error, @result);
+  my ($exit_code, $inner_error, $outer_error, $orig_pid, @result);
   {
+    $orig_pid = $$;
     local *STDIN = *CT_ORIG_STDIN if $localize{stdin}; # get original, not proxy STDIN
     # _debug( "# finalizing layers ...\n" );
     _relayer(\*STDOUT, $layers{stdout}) if $do_stdout;
     _relayer(\*STDERR, $layers{stderr}) if $do_stderr;
     # _debug( "# running code $code ...\n" );
-    local $@;
+    my $old_eval_err=$@;
+    undef $@;
     eval { @result = $code->(); $inner_error = $@ };
     $exit_code = $?; # save this for later
     $outer_error = $@; # save this for later
     STDOUT->flush if $do_stdout;
     STDERR->flush if $do_stderr;
+    $@ = $old_eval_err;
   }
   # restore prior filehandles and shut down tees
   # _debug( "# restoring filehandles ...\n" );
@@ -385,7 +399,7 @@ sub _capture_tee {
   # return captured output, but shortcut in void context
   # unless we have to echo output to tied/scalar handles;
   my %got;
-  if ( defined wantarray or ($do_tee && keys %localize) ) {
+  if ( $orig_pid == $$ and ( defined wantarray or ($do_tee && keys %localize) ) ) {
     for ( keys %do ) {
       _relayer($stash->{capture}{$_}, $layers{$_});
       $got{$_} = _slurp($_, $stash);
@@ -422,7 +436,7 @@ Capture::Tiny - Capture STDOUT and STDERR from Perl, XS or external programs
 
 =head1 VERSION
 
-version 0.29
+version 0.31
 
 =head1 SYNOPSIS
 
@@ -683,6 +697,14 @@ capturing:
 Attempting to modify STDIN, STDOUT or STDERR I<during> C<<< capture >>> or C<<< tee >>> is
 almost certainly going to cause problems.  Don't do that.
 
+B<Forking inside a capture>
+
+Forks aren't portable.  The behavior of filehandles during a fork is even
+less so.  If Capture::Tiny detects that a fork has occurred within a
+capture, it will shortcut in the child process and return empty strings for
+captures.  Other problems may occur in the child or parent, as well.
+Forking in a capture block is not recommended.
+
 =head2 No support for Perl 5.8.0
 
 It's just too buggy when it comes to layers and UTF-8.  Perl 5.8.1 or later
@@ -831,7 +853,7 @@ David Golden <dagolden@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Dagfinn Ilmari Mannsåker David E. Wheeler
+=for stopwords Dagfinn Ilmari Mannsåker David E. Wheeler Golden fecundf
 
 =over 4
 
@@ -843,6 +865,14 @@ Dagfinn Ilmari Mannsåker <ilmari@ilmari.org>
 
 David E. Wheeler <david@justatheory.com>
 
+=item *
+
+David Golden <xdg@xdg.me>
+
+=item *
+
+fecundf <not.com+github@gmail.com>
+
 =back
 
 =head1 COPYRIGHT AND LICENSE
